/*
  This program is free software published under the
  terms of the GNU Lesser General Public License.
  http://www.gnu.org/copyleft/lesser.html    
  This program is distributed in the hope that it will
  be useful - WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.
*/ 


/*  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
     coded in ECMAScript 5 - JavaScript 5  (ES5, ECMAScript 2009)
    xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx */


//  xxxxxxxxxxxxxxx   mst variable declarations  xxxxxxxxxxxxxxxx

// Note: let and const are not used because of backward compatibility

// These variables below can be directly modified by the user on the options form: 

var rows = 8;   // row count in one page (screen) => row count

var cols= 6;    //  column count in one page (screen) => column count

var itemCnt = 5;   // total item count (page count on options form)  => itemCnt= parseInt(document.getElementById("items").value, 10);    
 
var rand = false;    // random target frequency generator (Use full random generator for target) 

var pmin = 0;   // Minimum target frequency per page (percent)

var pmax = 25;  // Maximum target frequency per page (percent)
      
var parad = "3";   //  parad= ["0" => Match by word - black; "1" =>Match by word; "2" => Match by font; "3" => Match word and font]

var vtype = false;   // target stimulus - changing => [true => Random between pages or false => Constant ]

var tcolor = false;   // if (tcolor==true) => 16 basic html colors are used. Else Original Stroop colors are used 

var upcase = true;   // if (true) use uppercase letters for colors names; else use default (lowercase) color names 

var is_back = true; // if (is_back==true) => user can go back to previous item (Undo) 

var demo = true;     // if (demo == true) => show correct solution

var is_grid= true;  // Draw a border around each word (true) or not (false)

var timer = 120;     // Task time in seconds if clock = 1000ms (default). 

var prg_type = "1";   //  type of information in Progress Bar => default: remaining Time == "1". ("0" == items to end;  "2" == Nothing )

var store = false;   // false => this web application will not store results within client web browser.; true => This web application will store the results inside client web browser. 


// Other mst variables and constants:


var clock = 1000;  // basic timer resolution in ms. Default = 1s (1000ms)

var timer_id = null;   // timer_id is the identifier of the repeated action you want to cancel. This ID is returned from setInterval().

var counter = 0;      // timer counter. if (counter >= time) Timer will call theEnd()   

var item = 0;   // current item      

var dx;    // offset in the x-axis direction (for drawing cells)

var dy;   // offset in the y-axis direction (for drawing cells)

var dirty = false;  // if  (dirty==true) => user changed any options in form 

var is_task = false;    // task in progress or not  (for timer)
 
var fontStyle;        // canvas text font style 

/*  stim = jagged array= [item][word] =>
[
[0] = x; [1] =y; [2]= text; [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected
]*/
var stim = [];  

var stim_b = [];  // canvas (stimulus grid) bounds => stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom

var match = [];   // used only if target stimulus is changing between pages => for draw match (target) stimulus on header above grid in each page 

// default HTML Color Names strings. Color Names Supported by All Browsers. see: https://www.w3schools.com/colors/colors_names.asp
// 1935 year Stroop experiment: these colors were used: red, blue, green, brown, and purple (first 5 in array)   
var clr = ["red", "blue", "green", "brown", "purple", "yellow", "lime", "cyan", "magenta", "silver", "maroon", "olive", "teal", "navy", "pink", "orange" ];     

// user defined color names. 
var clr_c = null;  // each USED custom color is jagged array: [0=default html color name, 1=custom color name]

var max_ind= clr.length-1;  // max color arrays index for call function getColor()

var is_storage;       // if (true) => Web Storage Object is available

var custom_clr= false;  // if (true) => user defined color names will be used. Else: default html color names will be used in research

var custom_ins= false;  // if (true) => user defined instruction text content will be used. Else: default instructions text will be used in research



//  xxxxxxxxxxxxxxx     mst functions  xxxxxxxxxxxxxxxx



// init html page [create/start Timer, check storage, show start panel..] 
function init() 
{  
  if (timer_id)  {   
    window.clearInterval(timer_id);  
  }  
  timer_id= setInterval(function () {mstTimer()},clock);      
  if (isStorage() && storageAvailable('localStorage'))  {
    is_storage=true;
    document.getElementById("store_group").setAttribute("style", "display:block");  
    document.getElementById("no_storage").setAttribute("style", "display:none"); 
    var sins = localStorage.getItem('mst-v1.0-ins'); // Retrieve the instruction text content from storage
    if (sins)  {
      custom_ins= true;
      document.getElementById("custom_ins").setAttribute("style", "display:block");
      document.getElementById("custom_ins").textContent= sins;
      document.getElementById("def_ins").setAttribute("style", "display:none");      
    }   
  }
  else  {
    is_storage= false;
    document.getElementById("store_group").setAttribute("style", "display:none");  
    document.getElementById("no_storage").setAttribute("style", "display:block");      
  } 
  loadColors();        
  defPage();
  showStart();
}


// Load default html color names or user-defined color names into the "fixcolor" select element
function loadColors() 
{
  var done= false;
  var sel = document.getElementById("fixcolor"), opt;
  if (is_storage)  
  { 
    var tmp= localStorage.getItem('mst-v1.0-colors'); // Retrieve the custom colors names from storage. For each 16 colors jagged array: [index 0= custom color name; index 1 = use or not in research]
    if (tmp)  
    {
      tmp= JSON.parse(tmp);
      var len=tmp.length; 
      if (len > 4) 
      {    
        var j= 0;  // you must use new counter j
        for (var i= 0; i< len; i++) 
        {  
          if (tmp[i][1])  
          { // use this color in research (index 1)                          
            opt = document.createElement("option");
            opt.value = j;  // you must use new counter j            
            opt.text = tmp[i][0]; // user defined color name (index 0)
            //opt.setAttribute("data-color", clr[j]);   // default html color name
            sel.add(opt);  // sel.add(opt, null);              
          }
          j++;
        } 
        done=true; 
        document.getElementById("tcolor_msg").textContent= "Disabled: All user-selected colors will be used"; 
        document.getElementById("tcolor").selectedIndex=1; 
        document.getElementById("tcolor").disabled=true;
        tcolor= true;  //  true => use all user custom colors                    
        document.getElementById("custom_clr_msg").textContent= "User defined color names will be used";                    
      }
    }
  }
  if (!done)
  {
    var len=clr.length; 
    for (var i= 0; i< len; i++) 
    {                                       
      opt = document.createElement("option");
      opt.value = i.toString();
      opt.text = clr[i]; // default html color names
      sel.add(opt);  // sel.add(opt, null);        
    }  
  }  
  sel.selectedIndex=0;  
  
  // set window.custom_clr at same time..
  clr_c= null;  // reset to default 
  custom_clr= false;    // reset to default
  max_ind= clr.length-1;  // reset to default 
  if (is_storage)   
  {  
    // for each color name index 0 == color name,  index 1 == true/false use or not  
    if (tmp)  
    {    
      clr_c= [];  // init global array of custom colors names retrieved from storage 
      var j= 0;   // this is index (counter) from window.clr array
      var k= 0;   // this is index (counter) from window.clr_c array
      for (var i in tmp) 
      {  
        if (tmp[i][1]==true)  
        {   // use this color in research (index 1)
          clr_c[k]= [];  // for each USED custom color push jagged array: [0=default html color name, 1=custom color name]
          clr_c[k]= [clr[j], tmp[i][0]];   // [index 0 = clr[j] => default html color name, index = 1 => original custom color name from storage]            
          k++;
        }
        j++;
      } 
      if (clr_c.length > 4)  {
        custom_clr=true;  
        max_ind= clr_c.length-1;
      }      
    }
  }   
}


// set default page layout based on viewport/screen resolution
function defPage()
{
  var w= window.innerWidth, h= window.innerHeight, r;  
  if (w >= 2048)  {
    r= 16;  
  }
  else if (w >= 1024)  {
    r= 12;
  } 
  else if (w >= 512)  {
    r= 8;
  } 
  else   {
    r= 4;
  } 
  var c= Math.round(h/w * r);
  if (c >= 1 && c <= 50) {
    document.getElementById("rows").value= r;
    document.getElementById("cols").value= c;
    rows= r;
    cols= c;
    updCnt();
    setFreq(null);
  }    
} 


// show start panel 
function showStart()  
{      
  document.getElementById("end").style.display= "none";
  document.getElementById("google_wrapper").style.display="none";
  document.getElementById("table_grp").style.display="none";  
  document.getElementById("start").style.display= "block";
}


// before using web storage, check browser support for localStorage and sessionStorage
function isStorage()  {
  if (typeof(Storage) !== "undefined") {
     return true;               //  localStorage/sessionStorage is supported     
  } 
  else {
    return false;           // no browser Web Storage support
  }  
}


// necessarily because of IE11 and Edge offline mode
function storageAvailable(type) {
  try  {
    return (window[type]) ? true: false;   
  }
  catch(e) {
    return false;
  }
}


// show forms with application options (settings)
// <!-- IMPORTANT: showOption() and hideOption() buttons must be outside form element -->
function showOption()   
{
  document.getElementById("hide_option").style.display ="block";
  document.getElementById("options").style.display= "block";
  document.getElementById("start").classList.remove("child");  // don't work in <= IE 9    
  document.getElementById("start_div").style.display="none";  // .disabled = true;   
  document.getElementById("show_option").style.display ="none";  
  document.getElementById("hide_option").style="text-align:center";  // bug? auto-remove style when element.style.display="none"
  document.getElementById("instructions").style.display ="none";     
  if (document.getElementById("store_group").disabled === false && localStorage.getItem("mst-v1.0-data")) 
  {
    var cnt= JSON.parse(localStorage.getItem("mst-v1.0-data")).length;                                                        
    document.getElementById("store_cnt").textContent=  "You have a total of " + cnt  + " mst results stored in your web browser";
  }
  dirty= false;
}


// <!-- IMPORTANT: showOption() and hideOption() buttons must be outside form element -->
// save to window global vars all user Options/Settings
function hideOption()  
{
  setVarPar(); 
   
  document.getElementById("options").style.display="none"; 
  document.getElementById("start").classList.add("child");   
  document.getElementById("show_option").style.display ="block";   
  document.getElementById("start_div").style.display="block";        //.removeAttribute("disabled");  
  document.getElementById("hide_option").style.display ="none";   
  document.getElementById("start_div").style="text-align:center";  // bug?  (firefox auto remove class "center" when set style.display.none;)
  document.getElementById("show_option").style="text-align:center";   // bug?
  document.getElementById("instructions").style.display ="block";
  document.getElementById("ins_back").style.display = (is_back) ? "block" : "none";
   
  if (!custom_ins) {
    setDefIns();
  }
  
  refMsg();  
}


// setting (update) flexible default instructions text content..
function setDefIns()  
{  
  // display default instructions for selected paradigm 0..3 (hide unselected)  
  for (var i= 0; i< 4; i++)  {
    document.getElementById("ins_par" + i).style.display= (i == parad) ? "block" : "none";
  }
  document.getElementById("ins_fixed").style.display= "none";
  document.getElementById("ins_notfixed").style.display= "none";  
  if (parad != "3") 
  {  
    if (vtype) // vtype => target stimulus - changing => [true => Random between pages or false => Constant ]
    {
      document.getElementById("ins_notfixed").style.display= "block";
      if (parad === "0")  {
        document.getElementById("ins_notfixed").textContent="The target word will be different on each page. It will be displayed at the top of each page always in a black font.";
      }
      else if (parad === "1")  {
        document.getElementById("ins_notfixed").textContent= "The target word will change on each page. It is always displayed at the top of each page in a black font. You should click on all words that have the same text content as the target word at the top of each page while ignoring the font color.";   
      } 
      else if (parad === "2")  {
        document.getElementById("ins_notfixed").textContent= "The color of the target font is changeable and is printed at the top of each page in black font. On each page you need to find all the words that are printed in the target font color while ignoring the textual content of the words.";   
      }     
    }      
    else 
    {
      document.getElementById("ins_fixed").style.display= "block";     
      var txt= document.getElementById("fixcolor").options[document.getElementById("fixcolor").selectedIndex].textContent; // fixed color text
      var ind= document.getElementById("fixcolor").options[document.getElementById("fixcolor").selectedIndex].value;  // fixed html color name index
      var col= clr[ind];  
      if (parad === "2")  {
        document.getElementById("ins_fixed").innerHTML= 'You need to click on each word that has this <span style="font-size:x-large;font-weight:bold;background-color:white;color:' + col  + '">font color:' + txt + '</span>';        
      }
      else  {                                                                       
        document.getElementById("ins_fixed").innerHTML= 'You need to click on every word that has this <span style="font-size:x-large;font-weight:bold;background-color:white;color:black">textual content: ' + txt + '</span>';
      }
    }  
  }
  
  document.getElementById("ins_timer").textContent= "Remember that the duration of this task is always limited.";
  if (prg_type === "0")  {
    document.getElementById("ins_timer").textContent+=" At the bottom of the screen will be information on how many pages are left until the end of the task.";
  }
  else if (prg_type == "1")  {
     document.getElementById("ins_timer").textContent+= " The remaining time until the end of the task is shown to you at the bottom of the screen.";  
  }
  else  {
     document.getElementById("ins_timer").textContent+= " But information about the remaining time will not be visible";  
  }  
  document.getElementById("ins_demo").style.display= demo ? "block" : "none";
}

       
// refresh variable research Options (update *_msg label) => give user short option HELP
function refOption(e)  
{   
  dirty=true;       
  var my_id = e.id, msg = document.getElementById(my_id + "_msg"), s = "Values within this range are allowed: ", s2= " with step= ";    
  
  switch (my_id)
  {
    
    case "rows": 
    case "cols":    
    case "items":      
      if (e.validity.valid)  {           
        msg.removeAttribute("class");       
        msg.textContent="";               
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");    // msg.className = 'error'; 
      }       
      updCnt();
      break; 
                          
    case "rand":
      if (e.checked)  {
        msg.textContent="Yes";
        document.getElementById("pmin").disabled= true;
        document.getElementById("pmax").disabled= true;       
      }
      else  {
        msg.textContent="No";
        document.getElementById("pmin").disabled= false;
        document.getElementById("pmax").disabled= false;
      } 
      setFreq(null);        
      break;      
              
    case "pmin":
    case "pmax": 
      if (e.validity.valid)  {     
        setFreq(my_id);  
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step; 
        msg.setAttribute("class", "error");
      }        
      break;    
                                                 
    case "parad":            
      if (e.value === "0")  {    // black stimuli. target: word meaning (text content)
        msg.textContent= 'All stimuli have a black font color. The target stimulus is textual content (meaning of the matching word). The participant should simply click on each target word on each page.';       
      }
      else if (e.value === "1")  {    //  multiple colors stimuli. target: word meaning (text content). match by word only
        msg.textContent= 'The stimuli are shown in multiple colors. The target stimulus is only textual content (meaning of the word). The participant should click on each target word on each page. Eg. if the target word is "blue" participant should click on each word “blue” on the displayed page regardless of the font color.';               
      }
      else if (e.value === "2")  { // Match only by font color
        msg.textContent='The stimuli are shown in multiple colors. The target stimulus is only the font color. The participant should click on each word that has a target font color. Eg. if the target stimulus is "red" participant should click on each word that is printed in red font regardless of its textual content.';
      }
      else  {    // "3" => Match word and font
        msg.textContent='Target stimuli are all that have identical font color and textual content. Non-target stimuli have different color names and font colors. The participant should click on each word that has an identical font color and textual content (word meaning).';  
      }  
      document.getElementById("tcolor").disabled = custom_clr ? true : false; 
      document.getElementById("fixcolor").disabled = (e.value === "3" || document.getElementById("vtype").value != "0") ? true: false; 
      document.getElementById("vtype").disabled = (e.value === "3" ) ? true : false; 
      break;
      
    case "vtype":        
      msg.textContent= (e.value === "0") ? "The target stimulus will be the same during task" : "The target stimulus will be different on each page. It will be displayed at the top of each page always in a black font.";      
      document.getElementById("fixcolor").disabled = (e.value === "1" || document.getElementById("parad").value === "3") ? true: false;
      break;
                     
    case "tcolor":
      if (custom_clr)  {
        msg.textContent=  "All user-selected colors will be used";
      }
      else {
        msg.textContent= (e.value === "1") ? "All 16 html default colors will be used" : "Only these colors and words will be used: red, blue, green, brown and purple as in the classic 1935 Stroop experiment";
        document.getElementById("fixcolor_msg").textContent= (e.value === "0" && document.getElementById("fixcolor").selectedIndex > 4) ? "Warning: You have selected to use 5 Stroop colors and the selected fixed color does not belong to Stroop colors." : "This option is only available if you select:Target stimulus - changing: Constant and you may not choose:Match word and font";
      }        
      break;
    
    case "fixcolor":
      if (document.getElementById("tcolor").value === "0" && e.selectedIndex > 4) {
        msg.textContent= "Warning: You have selected to use 5 Stroop colors and the selected fixed color does not belong to Stroop colors.";   
      } 
      else {
        msg.textContent="This option is only available if you select:Target stimulus - changing: Constant and you may not choose:Match word and font";
      }
      break;
    
    case "upcase": 
      msg.textContent= (e.checked) ? "Uppercase letters for colors names will be used in research" : "Lowercase letters for colors names will be used in research"; 
      break;
    
    case "is_back":
    case "is_grid":
      msg.textContent= (e.checked) ?  "Yes" : "No";         
      break;
            
    case "demo":    
      if (e.checked)  {
        msg.textContent= " Yes - All target stimuli (words) will have this background color";
        msg.setAttribute("style", "background-color:#FFFFF0");   // background-color:ivory; border:red; border:1px solid #FF0000;
      }
      else  {
        msg.textContent=" No";
        msg.removeAttribute("style");
      }    
      break;
  
    // Temporal resolution          
    case "timer":
      if (e.validity.valid)  {
        msg.textContent= stringTime(e.value); 
        msg.removeAttribute("class");
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");
      } 
      break;                
             
    case "prg_type":                                                                                                                                            
      if (e.value === "0")  {
        msg.textContent= "It will be displayed how many pages (items) are left to the end.";
      }
      else if (e.value === "1")  {
        msg.textContent= "The remaining time until the end of the task will be displayed.";
      }
      else  {
        msg.textContent= "The timer will be on, but information on how much time is left until the end of the task will not be visible.";
      }
      break;   
      
    // storage 
    case "store":                                                                                                                                            
      msg.textContent= (e.value === "1") ? "This web application will store the results inside your web browser (no expiration date). Useful for group research" : "This web application will not store results within your web browser. Data is lost when the browser tab is closed";
      break; 
            
  }   // switch (my_id)
       
}


// update count of Total number of stimuli on one page, Total number of all stimuli in the task
function updCnt()  
{  
  if ( document.getElementById("rows").validity.valid && document.getElementById("cols").validity.valid && document.getElementById("items").validity.valid)         
  {  
    var t= parseInt(document.getElementById("rows").value, 10) * parseInt(document.getElementById("cols").value, 10);
    var e= t * parseInt(document.getElementById("items").value, 10);
    document.getElementById("page_cnt").textContent=  'Total number of stimuli on one page: ' + t; 
    document.getElementById("total_cnt").textContent= 'Total number of all stimuli in the task: ' + e;
    document.getElementById("page_cnt").removeAttribute("class"); 
    document.getElementById("total_cnt").removeAttribute("class");
  }
  else  {   
    document.getElementById("page_cnt").textContent= "'Total number of stimuli on one page: invalid value"; 
    document.getElementById("total_cnt").textContent="Total number of all stimuli in the task: invalid value";
    document.getElementById("page_cnt").setAttribute("class", "error");  
    document.getElementById("total_cnt").setAttribute("class", "error");     
  }   
}


// update Minimum and Maximum target frequency per page (percent)
// my_id == element.id or null
function setFreq(my_id)
{  
  if (document.getElementById("rand").checked || document.getElementById("pmin").disabled || document.getElementById("pmax").disabled)
  {
    document.getElementById("pmin_msg").textContent="";
    document.getElementById("pmax_msg").textContent=""; 
    return;
  }   
  if (parseInt(document.getElementById("pmin").value,10) <= parseInt(document.getElementById("pmax").value, 10))  
  {       
    document.getElementById("pmin_msg").removeAttribute("class");                   
    document.getElementById("pmin_msg").textContent= "Expected min target count= " + Math.round(parseInt(document.getElementById("pmin").value,10) * parseInt(document.getElementById("rows").value,10) * parseInt(document.getElementById("cols").value,10) / 100);   
    document.getElementById("pmax_msg").removeAttribute("class");
    document.getElementById("pmax_msg").textContent= "Expected max target count= " + Math.round(parseInt(document.getElementById("pmax").value,10) * parseInt(document.getElementById("rows").value,10) * parseInt(document.getElementById("cols").value,10) / 100);
  }
  else if (my_id) 
  {   
    var msg = document.getElementById(my_id + "_msg");
    msg.textContent= "Maximum target frequency per page must be greater or equal then Minimum target frequency per page"; 
    msg.setAttribute("class", "error");
  }
}


// refresh main validator message for user defined OPTIONS (feedback to user)
function refMsg()   
{ 
  if ( optionsValid() )  {
    document.getElementById("is_valid").textContent=  (dirty===true) ? "All your settings have been successfully updated" : "";  
    document.getElementById("is_valid").style="text-align:center;color:#000000"; 
  }
  else  {
    document.getElementById("is_valid").textContent="Some of your options are incorrectly set. They were not updated. These settings are marked in red."; 
    document.getElementById("is_valid").style= "text-align:center;color:#ff0000;font-weight:bold"; // safari 5.1.7 bug? => this don't work
  } 
}


// check for validity.valid object in all input type="number" (6x)
function optionsValid()  
{     
  var res= document.getElementById("rows").validity.valid  &&    
    document.getElementById("cols").validity.valid
    document.getElementById("items").validity.valid  &&          
    document.getElementById("pmin").validity.valid  &&
    document.getElementById("pmax").validity.valid  &&
    document.getElementById("timer").validity.valid;
    if (document.getElementById("rand").checked) {
      return res;
    }
    res = res && (parseInt(document.getElementById("pmin").value,10) <= parseInt(document.getElementById("pmax").value, 10));             
    return res;       // Firefox parser don't execute code ok without res var           
}


// start mst Task (start Research) 
function startTask(e)   
{       
  e.disabled= true;
  stimGenerator(); 
  e.disabled= false;   
  do  
    var ok= e.disabled;
  while (ok); // timer will wait for stimulus generator for large itemCnt 
  document.getElementById("google_wrapper").style.display="none";
  document.getElementById("end").style.display= "none"; 
  document.getElementById("start").style.display= "none"; 
  document.getElementById("canvas").style.display="block";                  
  document.getElementById("progress").style.display= "block";     
  document.getElementById("progress").setAttribute("style", "width:100%");   
  document.getElementById("time_left").textContent= "";
  
  if (document.getElementById("prg_type").value === "0") {
     document.getElementById("progress").className="progress-bar progress-bar-info progress-bar-striped";     // items/total
  }
  else if  (document.getElementById("prg_type").value === "1")  {
     document.getElementById("progress").className= "progress-bar progress-bar-danger progress-bar-striped";  // remaining time   
  }  
  document.getElementById("progress").setAttribute("style", "width:0");  // reset progress bar width
  
  counter= 0;       // reset timer counter
  is_task= true;    // finally activate timer
  
  drawStim();     // draw first page with stimulus   
}


// set application variable parameters
function setVarPar()  
{    
  
  if (document.getElementById("rows").validity.valid) rows= parseInt(document.getElementById("rows").value, 10);    
    
  if (document.getElementById("cols").validity.valid) cols= parseInt(document.getElementById("cols").value, 10);  
    
  if (document.getElementById("items").validity.valid) itemCnt= parseInt(document.getElementById("items").value, 10);        
  
  rand= document.getElementById("rand").checked ? true: false;    // random target frequency generator (Use full random generator for target)
    
  if (document.getElementById("pmin").validity.valid && document.getElementById("pmax").validity.valid 
    && (parseInt(document.getElementById("pmin").value,10) <= parseInt(document.getElementById("pmax").value, 10) ) )   {  
      pmin= parseInt(document.getElementById("pmin").value, 10); 
      pmax= parseInt(document.getElementById("pmax").value, 10); 
  }
  
  parad = document.getElementById("parad").value;     //  parad= ["0" => Match by word - black; "1" =>Match by word; "2" => Match by font; "3" => Match word and font]
 
  vtype = (document.getElementById("vtype").value === "1") ? true: false;    // target stimulus - changing => [true => Random between pages or false => Constant ]
                      
  tcolor= (document.getElementById("tcolor").value === "0") ? false: true;  // false => Original Stroop colors; true => use basic html colors    
  
  upcase= document.getElementById("upcase").checked ? true: false;    // uppercase or lowercase color names
  
  is_back= document.getElementById("is_back").checked ? true: false;   // enable Go back (previous item)      
    
  demo= document.getElementById("demo").checked ? true: false; // demo mode = true => show correct solution
  
  is_grid= document.getElementById("is_grid").checked ? true: false; //  Draw a border around each word (cell) or not
    
  if (document.getElementById("timer").validity.valid) timer= Math.round(parseInt(document.getElementById("timer").value, 10));  
  
  prg_type= document.getElementById("prg_type").value;   //  type of information displayed on Progress Bar   
    
  store= (is_storage === true && document.getElementById("store").value === "1") ? true : false;
        
}


// set internal application research parameters
function setPar()  
{            
  stim_b = [];  // bounds for stimulus
  stim = [];    // [item][shape] => [0] = x; [1] =y;  [2]= text; [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected    
  match = [];   // match stimulus (target)
  item= 0;  

  // update uppercase (lowercase)  
  if (custom_clr)  {  
    for (var i in clr_c)  {  // each USED custom color is jagged array: [0=default html color name, 1=custom color name]        
       clr_c[i][1]= upcase ? clr_c[i][1].toUpperCase() : clr_c[i][1].toLowerCase();   // [index 0 => default html color name, index 1 => custom color name from storage]            
    }
  }
  else {
    for (var i in clr)  {
      clr[i]= upcase ? clr[i].toUpperCase() : clr[i].toLowerCase();
    }     
  }     
}


// main stimulus generator function
function stimGenerator()  
{ 
  setPar();      
  
  setPage();
    
  var i= 0, b= false, ind, prev, txt;    // i = ordinal item count [0..itemCnt-1]   
  var fixed= document.getElementById("fixcolor").selectedIndex;  // selected color index     
  for (var j=0; j<itemCnt; j++)
  {  
    if (!b || vtype) 
    {  // vtype => target stimulus - changing => [false => Constant or true == Random between pages]
      if (vtype)  
      {
        do {
          ind= tcolor ? getRndInt(0,max_ind) : getRndInt(0,4);
        } while (ind === prev);  
      }
      else  {
        ind=fixed;  
      }      
      b= true;    
    }
     
    stim[i]= [];   
    if (parad == "3")  {
        stim[i]= setStim3(stim_b[0], stim_b[1], stim_b[2], stim_b[3]);             
    }
    else  {
      stim[i]= setStim02(stim_b[0], stim_b[1], stim_b[2], stim_b[3], ind);  
    }     
    match[i]= getColor(ind, false);   // remember match color text content for this item (page)  
    prev= ind;
    i++;    // next item
  }            
}


/* set canvas (viewport) choices (options) bounds        
   stim_b array =>  [0] = left; [1] = top; [2] = right; [3]= bottom  */                                                                                    
function setPage()  
{     
  document.getElementById("footer").style.display="block";  
  document.getElementById("footer").height= "50";  
  var canvas= document.getElementById("canvas");     
  canvas.left= 0; 
  canvas.top= 0;
  canvas.width= window.innerWidth;       // stretch canvas to parent
  canvas.height= window.innerHeight - document.getElementById("footer").height;                                                  
  // set viewport cells raster (dx, dy) 
  dx = Math.floor(canvas.width/cols);  
  dy = (vtype && parad != "3") ? Math.floor(canvas.height/(rows+1)) : Math.floor(canvas.height/rows);    // if (vtype) add 1 row for header 

  //  stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom      
  stim_b= (vtype && parad != "3") ? [0, dy, canvas.width, canvas.height] :  [0, 0, canvas.width, canvas.height];                                                                                                                  
   
  if (Math.floor(Math.min(dx,dy)/2) <= 5) alert("Your screen resolution is probably not enough to distinguish the drawings that will be displayed.\nPossible solutions:\n  -reduce Row count to a smaller value Currently= " + rows + "\n  -reduce the Column count. Currently= " + cols);      

  setFont();   
}
   

// todo: this function is not the smartest. All suggestions and improvements are more than welcome.. 
function setFont()
{ 
  // adjust the optimal font horizontally
  var canvas= document.getElementById("canvas");    
  var ctx = canvas.getContext('2d'); 
  var w_max= 0, w_tmp, s;  
  var tmp= custom_clr ? clr_c : clr; 
  for (var i in tmp)  {
    w_tmp= ctx.measureText(tmp[i]).width;    
    if (w_tmp > w_max) {
      w_max= w_tmp;
      s= tmp[i];  // x-longest color name (default names or custom colors names)
    }  
  }    
  var size=window.innerWidth/2;  
  do  
  {
    size--;
    ctx.font= size + "px Georgia";
  } while (ctx.measureText(s).width >= dx) ;
  
  size= Math.min(size,dy);   // adjust the optimal font vertically  
  //size= Math.round(0.9*Math.min(size,dy));
  fontStyle= size + "px Georgia";   
}


/* set match and sample coordinates; generate stim-coord (x,y) and default properties (target/non-target, color, shapes type)
 divide screen section into raster equal segments (x-horizontal and y-vertical) 
 return array  stim coordinates =>  array index: [0] = x (left); [1] =y (top);  [2]= shape type;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected
 parameters:   stim_b array =>  [0] = _left; [1] = _top; [2] = _right; [3]= _bottom 
               ind = index of color (or color name) for target stimulus */     
function setStim02(_left, _top, _right, _bottom, ind) 
{    
  var y=_top, x, max, low, high, cnt, pos, sel= {}, col, txt, tmp, is_target, arr= [];   
  
  // select target random cells in range low..hight
  max = rows*cols-1;
  if (rand)  {
    low= 0;    // true random generator
    high=rows*cols; 
  }
  else {
    low= Math.round(pmin * rows * cols / 100);
    high = Math.round(pmax * rows * cols / 100); 
  }
  cnt = getRndInt(low,high); 
  for (var i= 1; i<= cnt; i++)
  {
    do
      pos= getRndInt(0, max);
    while (typeof sel[pos] != "undefined");
    sel[pos]= pos;  
  }     
  // set cells content (font and text)
  pos= 0;  //  parad= ["0" => Match by word - black; "1" =>Match by word; "2" => Match by font; "3" => Match word and font]
  for (var i=1; i<=rows; i++) 
  { // i == screen row == x     
    x=_left;
    for (var j=1; j<=cols; j++)  
    {  // j == screen column == y                                               
      do  { 
        tmp= tcolor ? getRndInt(0,max_ind) : getRndInt(0,4);   // find new color index different from target stimulus                                         
      } while (tmp === ind);  // must be different from target index
                  
      if (typeof sel[pos] === 'undefined')
      { // non-target stimulus => change text and font. all nontarget stimuli are different from target stimuli and have different text + font color 
        is_target= false;     
        if (parad === "0")  {
          txt= getColor(tmp,false); // nontarget stimulus: change printed text. text must be different from target stimulus
          col= "#000000";  // black font color 
        }  
        else if (parad === "1")  {
          txt= getColor(tmp,false); // nontarget stimulus: text must be different from target stimulus
          tmp= tcolor ? getRndInt(0,max_ind) : getRndInt(0,4);   // but nontarget font color can be equal as target text content
          col= getColor(tmp,true);   // nontarget stimulus: font color can be equal as target text
        }  
        else /*if (parad === "2") */ {
          col= getColor(tmp,true);   // nontarget stimulus: font color must be different from target font color
          tmp= tcolor ? getRndInt(0,max_ind) : getRndInt(0,4);   // but nontarget text content can be equal as target font color
          txt= getColor(tmp,false); // nontarget stimulus: text can be equal as target font color                     
        }        
      }                                     
      else  
      { // this is target stimulus section
        is_target= true;   // target stimulus text == font color         
        if (parad === "0")  {  // "0" => Match by word - black;
          txt= getColor(ind,false); // text content (target color name). we don't change it 
          col= "#000000";      // print target always in black color (this is font color)
        } 
        else if (parad === "1")  {  // "1" => Match by word;          
          txt= getColor(ind,false);  // this is target text content (word, printed text). we don't change it        
          col= getColor(tmp,true);   // we will now change target font color for target stimlui (font color now will be different from text)
        }
        else /*if (parad === "2")*/  {  // "2" => Match by font
          txt= getColor(tmp,false); // we will now change printed text. It will be different from target font color
          col= getColor(ind,true);   // this is target font color. we don't change it         
        }
      }
      arr.push([x, y, txt, col, is_target, false]);   // array index: [0]=x (left); [1]=y (top); [2]= text;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected      
      x+=dx; // new shape within row => add x
      pos++;    
    } 
    y+=dy;   // new row (add y)
  } 
  return arr;  
}


/* set match and sample coordinates; generate stim-coord (x,y) and default properties (target/non-target, color, shapes type)
 divide screen section into raster equal segments (x-horizontal and y-vertical) 
 return array  stim coordinates =>  array index: [0] = x (left); [1] =y (top);  [2]= shape type;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected
 parameters:   stim_b array =>  [0] = _left; [1] = _top; [2] = _right; [3]= _bottom  */     
function setStim3(_left, _top, _right, _bottom) 
{    
  var y=_top, x, max, low, high, cnt, pos, sel= {}, ind, col, txt, is_target, arr= [];   
  // select target random cells in range low..hight
  max = rows*cols-1;
  if (rand)  {
    low= 0;    // true random generator
    high=rows*cols; 
  }
  else {
    low= Math.round(pmin * rows * cols / 100);
    high = Math.round(pmax * rows * cols / 100); 
  }
  cnt = getRndInt(low,high); 
  for (var i= 1; i<= cnt; i++)
  {
    do
      pos= getRndInt(0, max);
    while (typeof sel[pos] != "undefined");
    sel[pos]= pos;  
  }   
  // set cells content (font and text)
  pos= 0;
  for (var i=1; i<=rows; i++) 
  { // i == screen row == x     
    x=_left;
    for (var j=1; j<=cols; j++)  
    {  // j == screen column == y  
      ind = tcolor ? getRndInt(0,max_ind) : getRndInt(0,4);   // get new random color index                                  
      col= getColor(ind,true);      // font color
      txt= getColor(ind,false);    // cell text content 
      if ( typeof sel[pos] === 'undefined')
      { // non-target stimulus => change text content
        is_target= false;     
        do  { 
          txt= tcolor ? getRndInt(0,max_ind) : getRndInt(0,4);                                            
        } while (txt === ind);  
        txt= getColor(txt,false);  // override => text is now different from font color     
      }                                     
      else  
      { // target stimulus => text == font color             
        is_target= true;   // target stimulus text == font color            
      }
      arr.push([x, y, txt, col, is_target, false]);   // array index: [0]=x (left); [1]=y (top); [2]= text;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected      
      x+=dx; // new shape within row => add x
      pos++;
    } 
    y+=dy;   // new row (add y)
  } 
  return arr;  
}


/* sets one of mostly basic html predefined colors. this is mostly default set of 16 html Basic colors 
 par: ind == array index (for global window.clr or window.clr_c arrays)
 par: is_clr => is used ONLY for custom color names. 
 if (is_clr) return html (default) color name (for font color); else return custom color names (for text content) */
function getColor(ind, is_clr) 
{
  if (custom_clr) 
  {
    if (is_clr)   {
      return clr_c[ind][0];  // html color name (for font). index 0
    }
    else  {
      return clr_c[ind][1];  // custom color name string (for text content). index 1
    }  
  }
  else  {  
    return clr[ind];  // default: font color names == text content
  }
}

 
// get random integer from min to max range
function getRndInt(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}


// application timer handler function
function mstTimer()  
{      
  if (!is_task) {
    return;  // nop, idle        
  }
  counter++;
  if (counter >= timer)  {     // task time is limited
    theEnd();
  }     
  else if (prg_type === "1")  {   // "1" => Time. display time on Progress Bar  
    var rest=timer-counter;   // this is remaining time 
    var percent= Math.round(100*rest/timer);    
    document.getElementById("progress").setAttribute("style", "width:"+percent+"%");    // !important
    document.getElementById("time_left").textContent= stringTime(rest);    
  }   
} 


// return integer into string hh:mm:ss time format
// input par: total == integer (time in seconds)
function stringTime(total) 
{
  var hh = Math.floor(total/(60*60));         
  var divisor_for_minutes = total % (60 * 60);
  var mm = Math.floor(divisor_for_minutes / 60);       
  var divisor_for_seconds = divisor_for_minutes % 60;
  var ss = Math.ceil(divisor_for_seconds);    
  // These lines ensure you always have two-digits. This formats your string to HH:MM:SS
  if (hh < 10)  {hh = "0"+hh;}     
  if (mm < 10) {mm = "0"+mm;}
  if (ss < 10) {ss = "0"+ss;} 
  return hh + ":" + mm + ":" + ss;
} 


// goto previous item (stimulus)
function priorAnswer()  
{                         
  if (item > 0)  {
    item--;  
    drawStim();     // draw again previous stimulus
  }         
}


// goto next item (stimulus)
function nextItem()  
{               
  if (item < itemCnt-1)  {         
    item++; 
    drawStim();     // draw next stimulus        
  }
  else  {               
    theEnd();  // end of task  
  }
}


// draw (repaint) stimulus    
function drawStim() 
{    
  var canvas= document.getElementById('canvas');
  var ctx = document.getElementById('canvas').getContext('2d'); 
  ctx.clearRect(0, 0, canvas.width, canvas.height);      // clear canvas             
  
  if (vtype && parad != "3")  {  // draw match (target) stimulus above stim_b rect if exists
    drawText(ctx, Math.floor((canvas.width-dx)/2),  stim_b[1]-dy, match[item], "#000000");   // stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom    
  }
  
  for (var s in stim[item])   
  {   // s= stimulus number                                           
    ctx.beginPath();     // this section draw square border for each cell => rectangle 
    ctx.rect(stim[item][s][0], stim[item][s][1], dx, dy);   //  	context.rect(x,y,width,height);
    ctx.strokeStyle=  "#000000";     
    if (stim[item][s][5] || (demo && stim[item][s][4]) === true)    // [4]= target or not; [5] == selected or not 
    { // set background color for selected answer or set background color in demo mode for correct answer. you must call before drawShape() handler
      // Gray90 == selected or Ivory == correct 
      ctx.fillStyle=  stim[item][s][5]   ?  "#E5E5E5" : "#FFFFF0";  // the selected answer takes precedence over the correct answer (background color)                        
      //ctx.strokeStyle= "#FF0000";
    }
    else  {
      ctx.fillStyle= "#FFFFFF";
      ctx.strokeStyle= "#000000";
    }  
    ctx.fill();             
    if (is_grid) { 
      ctx.stroke();
    }
    drawText(ctx, stim[item][s][0], stim[item][s][1], stim[item][s][2], stim[item][s][3]);  // draw words in grid cells (target and non-target stimulus)
  }  
  // set footer navigation prior button (left arrow) 
  if (item > 0 && is_back)  {  
    document.getElementById("prior").style.display= "inline"; 
  }
  else  {
    document.getElementById("prior").style.display= "none";   
  } 
  // update "Next item" image on "right arrow" button 
  document.getElementById("next").style.backgroundImage = (item === itemCnt-1)  ?  "url('img/end.png')" : "url('img/forward.png')"; 
  // display (update) item count on bottom Progress Bar   
  if (prg_type === "0" )  {                                                                                                                                                         
    var percent = 100*(itemCnt-(item+1))/itemCnt;  // Math.round();      
    document.getElementById("progress").setAttribute("style", "width:"+percent+"%");    // !important
    document.getElementById("time_left").textContent= "Page= " + (item + 1) + " Remaining= " + (itemCnt - item-1);  
  } 
}


/* draw text on canvas
    ctx = canvas (object) context,
    x = x-left,
    y = y-top,
    txt = text 
    col = color (fillStyle)  
*/  
function drawText(ctx, x, y, txt, col)
{ 
  x+=dx/2;    
  y+=dy/2;     
  ctx.textAlign = "center";
  ctx.textBaseline = "middle";
  ctx.font = fontStyle;  
  ctx.fillStyle = col;  // line color 
  ctx.fillText(txt, x, y);                               
}  


// user selected shape on canvas 
//<canvas id="canvas" onclick="doAnswer(event)"></canvas>   
function doAnswer(e)  
{        
  var x= e.clientX, y= e.clientY; 
  // stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom 
  if (x > stim_b[0] && x < stim_b[2] && y > stim_b[1] && y < stim_b[3])  
  {      
    var row = Math.ceil((y - stim_b[1])/dy) -1;    
    var col = Math.ceil((x - stim_b[0])/dx)- 1;   
    if (row <= rows && col < cols)  
    {
      var p = row * cols + col; // shape position within stim[item] array (page)
      var is_correct= stim[item][p][4];          
      if (stim[item][p])
      {   // array index: [0] = x; [1] =y;  [2]= shape type;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected
        stim[item][p][5]= (stim[item][p][5] === false) ? true : false;   // toggle (reset) answer if user choose already selected answer        
        drawStim(); // repaint => update backgroundColor for (un)selected shapes
      }         
    }     
  }   
}


// end of task (research)
function theEnd()  
{     
  is_task= false;
  document.getElementById("canvas").style.display="none"; // this disable doAnswer() event
  document.getElementById("footer").style.display="none";
  document.getElementById("end").style.display="block";  
  var res=0, err=0, mis=0, tot=0;  // res== correct answers; err= errors, mis= missing values; tot= maximum possible score

  //stim[item][s]  => [ [0] = x; [1] =y;  [2]= text; [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected  ]
  for (var i in stim)   
  {  // i == item
    for (var s in stim[i])   
    {  // s== shape
      if (stim[i][s][4] && stim[i][s][5]) { 
        res++;   // target and selected
        tot++;
      }
      else if (stim[i][s][4] && !stim[i][s][5] )  {
        mis++;       // target and not selected
        tot++;
      }
      else if (!stim[i][s][4] && stim[i][s][5] )  {
        err++;     // not-target and selected
      }
    }
  }                                                            
  document.getElementById("score").textContent= "Correct= " + res + " Missing= " + mis + " Errors= " + err + " Maximum possible score=" + tot;
  document.getElementById("no_google").textContent="";  
  document.getElementById("chart_title").style.display="none"; 

  try  {
    drawReport(res,err,mis);  
  }
  catch (e)  {
    document.getElementById("no_google").textContent= "You must have an available internet connection to display graphs and tables. Make sure you are online."; 
  } 
  
  try 
  {
    if (store) {
      drawStorage(res,err,mis,tot);  
    }
  }        
  catch (e)  { 
    document.getElementById("table_grp").innerHTML= "Your browser probably doesn't have support for the Web Storage Object or access to the Storage Object is disabled. Check your browser settings. In some browsers (IE11, Edge) Storage Object is not enabled when accessed in a local file (offline)."; 
  }
           
}


/* async draw and display table and chart with user results (call google API)..
  (chart = PIE + table) => Displays the sum of: correct answers, errors, and missing values  
  res = total correct answers in task (sum)
  err = total errors in task (sum)
  mis = total missing values in task (sum) = Number of target stimuli not detected by the user */
function drawReport(res, err, mis) 
{          
  document.getElementById("google_wrapper").style.display="block";  
  //    xxx  table = Displays the sum of: correct answers, errors, and missing values  xxx 
  var data = new google.visualization.DataTable();    // Create and populate the data table. 
  data.addColumn('string', 'Answer type');     // 1. col = ans array = must be string  
  data.addColumn('number', 'Sum');      // sum(correct answers, errors, missing values) = number = frequencies             
  data.addRow ( ["Correct", res] );
  data.addRow ( ["Errors", err] ); 
  data.addRow ( ["Missing", mis] );                                      
  options= {      
    'width': '50%',    
    'allowHtml': true, 
    'sort': 'enable',  
    'alternatingRowStyle': true              
  };           
  var elem= document.getElementById("table");
  elem.style.display = "block"; 
  new google.visualization.Table(elem).draw(data, options); 
  //    xxxxxx      PIE chart  => data are same   => draw PIE chart and table for correct, errors, missing values   xxxxxx 
  options= {   
   is3D: true, 
   legend: 'none',
   pieSliceText: 'value',
   tooltip: {text:'value'},    
   width: '100%',
   chartArea: {width:'100%'}
  };          //title: 'Correct answers, errors and missing values',   
  elem = document.getElementById("chart");   
  elem.style.display = "block"; 
  new google.visualization.PieChart(elem).draw(data, options);     // Create and draw the visualization.     
  document.getElementById("chart_title").style.display="block";  
}


/*  use window.localStorage in client browser
    res = total correct answers in task (sum)
    err = total errors in task (sum)
    mis = total missing values in task (sum)
    tot= maximum possible score in task  */
function drawStorage(res,err,mis,tot)   
{  
  document.getElementById("table_grp").style.display="block";
  var oldItems = JSON.parse(localStorage.getItem("mst-v1.0-data")) || []; 
  var id= oldItems.length+1;     
  var fixed= ""; // fixed color name
  if (parad != "3" && vtype === false /*&& !document.getElementById("fixcolor").disabled */) { 
    fixed= document.getElementById("fixcolor").options[document.getElementById("fixcolor").selectedIndex].textContent; // fixed color text
  }
  // set new "record" 
  var newItem = {
    'id': id,
    'date': new Date(),      
    'correct': res, 
    'errors': err,       
    'missing': mis,   
    'max-score': tot,
    'show-correct' : (demo) ? "Yes": "No",      
    'timer': timer,
    'stimulus-per-Page' : rows*cols,
    'pages': itemCnt,      
    'min-frequency': rand ? "": pmin,
    'max-frequency': rand ? "": pmax,     
    'match-type' : parad,      
    'target-changing' : vtype,       
    'colors' : document.getElementById("tcolor").selectedIndex,
    'const-color' : fixed,
    'is-back' : is_back ? "Yes": "No"         
  };
  
  oldItems.push(newItem);

  localStorage.setItem("mst-v1.0-data", JSON.stringify(oldItems));

  createTableJSON(document.getElementById("table_grp"), oldItems); 
   
  document.getElementById("score").textContent+= ". (ID=" + id + ")";
 
}


// convert JSON data dynamically to HTML Table using JavaScript - Dynamically Create table with rows in JavaScript
// downloaded and adapted from source: https://www.encodedna.com/javascript/populate-json-data-to-html-table-using-javascript.htm
function createTableJSON(elem, my_json) 
{  
  // EXTRACT VALUE FOR HTML HEADER. 
  var col = [];
  for (var i = 0; i < my_json.length; i++) {
    for (var key in my_json[i]) {
        if (col.indexOf(key) === -1) {
            col.push(key);
        }
    }
  } 
  var table = document.createElement("table");
  table.setAttribute("class", "my_table");    // table.className = 'my_table';
  // CREATE HTML TABLE HEADER ROW USING THE EXTRACTED HEADERS ABOVE.
  var tr = table.insertRow(-1);                   // TABLE ROW.
  for (var i = 0; i < col.length; i++) {
      var th = document.createElement("th");      // TABLE HEADER.
      th.innerHTML = col[i];
      tr.appendChild(th);
  }
  // ADD JSON DATA TO THE TABLE AS ROWS.
  for (var i = 0; i < my_json.length; i++) 
  {
    tr = table.insertRow(-1);
    for (var j = 0; j < col.length; j++) {
      var tabCell = tr.insertCell(-1);
      tabCell.innerHTML = my_json[i][col[j]];
    }
  }
  // FINALLY ADD THE NEWLY CREATED TABLE WITH JSON DATA TO A CONTAINER.
  elem.innerHTML= "";
  elem.appendChild(table);
}


// clear mst web browser storage data
function clrStorage()  
{  
  var cnt= localStorage.getItem("mst-v1.0-data")  ?  JSON.parse(localStorage.getItem("mst-v1.0-data")).length : 0;                                                        
  if (cnt === 0)  {
    alert("You have no mst data to delete");
    return;
  }  
  if ( !window.confirm("You have a total of " + cnt  + " mst results stored in your web browser. Continue?") ) { 
   return;
 }                                             
 if (window.confirm("Delete total saved mst results:  "  + cnt + ". This action cannot be undone. Final confirmation?") ) 
 { 
   window.localStorage.removeItem("mst-v1.0-data"); 
   document.getElementById("store_cnt").textContent= "All data has been deleted. No data";
 }  
}
